@@ -1,7 +1,7 @@
 context("values")
 
 test_that("values", {
-  ps = ParamSet$new(list(
+  ps = ParamSet_legacy$new(list(
     ParamDbl$new(id = "d", lower = 0, upper = 1),
     ParamInt$new(id = "i", lower = 1, upper = 3),
     ParamFct$new(id = "f", levels = letters[1:3])
@@ -12,24 +12,24 @@ test_that("values", {
   ps$values = list(d = 1, f = "a")
   expect_true(ps$check(list(d = 0, f = "a")))
   ps2 = ps$clone()
-  ps2$subset(ids = c("d", "i"))
+  ps2 = ps2$subset(ids = c("d", "i"))
   expect_equal(ps2$values, list(d = 1))
   ps2$values = list(d = 0.5)
   expect_true(ps$check(list(d = 1, f = "a")))
   expect_equal(ps2$values, list(d = 0.5))
   # check printer
-  expect_output(print(ps2), "d.*<NoDefault(?:\\[3\\])?>.*0.5")
+  expect_output(print(ps2), "d.*<NoDefault\\[0\\]>.*0.5")
 
   ps2 = ps$clone()
-  ps2$subset(ids = c("i"))
+  ps2 = ps2$subset(ids = c("i"))
   expect_equal(ps2$values, set_names(list(), character(0)))
 
-  ps3 = ParamSet$new(list(
+  ps3 = ParamSet_legacy$new(list(
     ParamDbl$new(id = "x", lower = 0, upper = 9)
   ))
   ps3$values = list(x = 7)
   ps2 = ps$clone()
-  ps2$add(ps3)
+  ps2 = ps_union(list(ps2, ps3))
   expect_equal(ps2$values, list(d = 1, f = "a", x = 7))
 
   # designs
@@ -62,7 +62,7 @@ test_that("values", {
 test_that("values calls assert", {
   # most of the tests should be done for ParamSet$check, so we simply
   # check here, that paramvals calls assert
-  ps = ParamSet$new(list(
+  ps = ParamSet_legacy$new(list(
     ParamDbl$new(id = "d", lower = 0, upper = 1),
     ParamInt$new(id = "i", lower = 1, upper = 3),
     ParamFct$new(id = "f", levels = letters[1:3])
@@ -72,8 +72,8 @@ test_that("values calls assert", {
 
   # now check that we can disable assert
   ps$assert_values = FALSE
-  ps$values = list(xxx = 1)
-  expect_equal(ps$values, list(xxx = 1))
   ps$values = list(d = 9)
   expect_equal(ps$values, list(d = 9))
 })
+
+
